"""Use getdependentfiles.py in a batch of tests"""

# This sample uses the getdependentfiles.py script to try to get the
# dependents of some predetermined files.

import os
import sys

try:
    import cx_Freeze
except ImportError:
    print("Please install a cx-freeze package to test", file=sys.stderr)
    sys.exit(-1)

program = f"{sys.executable} getdependentfiles.py"
dlls = os.path.join(sys.base_prefix, "DLLs")
scripts = os.path.join(sys.base_prefix, "Scripts")
cx_Freeze_dir = os.path.dirname(cx_Freeze.__file__)

dependencies_to_check = [
    os.path.join(sys.base_prefix, "python.exe"),
    os.path.join(
        sys.base_prefix,
        "python{}{}.dll".format(sys.version_info[0], sys.version_info[1]),
    ),
    sys.executable,
    os.path.join(dlls, "_ctypes.pyd"),
    os.path.join(dlls, "_sqlite3.pyd"),
    os.path.join(dlls, "_ssl.pyd"),
    os.path.join(scripts, "pip.exe"),
    os.path.join(cx_Freeze_dir, "bases", "Console.exe"),
]

print("testing with default PATH environ")
cmdline = program + " " + " ".join(dependencies_to_check)
for line in os.popen(cmdline):
    print(line, end="")
print()

print("testing with sys.path into PATH environ")
os.environ["PATH"] = (
    os.pathsep.join(sys.path) + os.pathsep + os.environ["PATH"]
)
cmdline = program + " " + " ".join(dependencies_to_check)
for line in os.popen(cmdline):
    print(line, end="")
print()



# A very simple setup script to create a single executable that makes use of
# the openpyxl package. This package, like a number of others, makes the
# assumption that it is found in the file system, and so fails miserably if
# it is included in a zip file.
#
# Run the build process by running the command 'python setup.py build'
#
# If everything works well you should find a subdirectory in the build
# subdirectory that contains the files needed to run the script without Python

from cx_Freeze import setup, Executable

executables = [Executable("test_openpyxl.py")]

setup(
    name="test_openpyxl",
    version="0.1",
    description="Sample cx_Freeze script testing the use of openpyxl",
    executables=executables,
)



# A very simple setup script to create a single executable, and test the BUILD_CONSTANTS functionality.
#
# hello.py is a very simple 'Hello, world' type script which also displays certains constants stored in
# BUILD_CONSTANTS
#
# Run the build process by running the command 'python setup.py build'
#
# If everything works well you should find a subdirectory in the build
# subdirectory that contains the files needed to run the script without Python

from cx_Freeze import setup, Executable

executables = [Executable("hello.py")]

options = {"build_exe": {"constants": ["A=7", 'B="hello=7"']}}


setup(
    name="hello",
    version="0.1",
    description="Sample cx_Freeze script",
    options=options,
    executables=executables,
)



# A simple setup script to create an executable using PySide2. This also
# demonstrates how to use include_files, zip_include_packages and excludes
# to get minimal package size
#
# PySide2app.py is a very simple type of PySide2 application
#
# Run the build process by running the command 'python setup.py build'
#
# If everything works well you should find a subdirectory in the build
# subdirectory that contains the files needed to run the application
import os
import sys

import PySide2
from cx_Freeze import setup, Executable

plugins_path = os.path.join(PySide2.__path__[0], "plugins")


base = None
if sys.platform == "win32":
    base = "Win32GUI"

options = {
    "build_exe": {
        "include_files": [
            os.path.join(plugins_path, "platforms")
        ],  # additional plugins needed by qt at runtime
        "zip_include_packages": [
            "PySide2",
            "shiboken2",
            "encodings",
        ],  # reduce size of packages that are used
        "excludes": [
            "tkinter",
            "unittest",
            "email",
            "http",
            "xml",
            "pydoc",
            "pdb",
        ],  # exclude packages that are not really needed
    }
}

executables = [Executable("PySide2app.py", base=base)]

setup(
    name="simple_PySide2",
    version="0.1",
    description="Sample cx_Freeze PySide2 script",
    options=options,
    executables=executables,
)



"""A simple script to show some values of a font in matplotlib."""

from pathlib import Path
import matplotlib as mpl
from matplotlib.afm import AFM

print("matplotlib datapath:", mpl.get_data_path())
afm_path = Path(mpl.get_data_path(), "fonts", "afm", "ptmr8a.afm")
with afm_path.open("rb") as fh:
    afm = AFM(fh)
print(
    "compare the results with:",
    "https://github.com/matplotlib/matplotlib/blob/master/lib/matplotlib/afm.py",
)
print(afm.string_width_height("What the heck?"))
print(afm.get_fontname())
print(afm.get_kern_dist("A", "f"))
print(afm.get_kern_dist("A", "y"))
print(afm.get_bbox_char("!"))



"""
This module contains utility functions shared between cx_Freeze modules.
"""

import types


def normalize_to_list(value) -> list:
    """
    Takes the different formats of options containing multiple values and
    returns the value as a list object.
    """
    if value is None:
        normalized_value = []
    elif isinstance(value, str):
        normalized_value = value.split(",")
    else:
        normalized_value = list(value)

    return normalized_value


def rebuild_code_object(co, code=None, constants=None, filename=None):
    """Rebuild the code object."""
    code = code or co.co_code
    constants = tuple(constants or co.co_consts)
    filename = filename or co.co_filename
    params = [
        co.co_argcount,
        co.co_kwonlyargcount,
        co.co_nlocals,
        co.co_stacksize,
        co.co_flags,
        code,
        constants,
        co.co_names,
        co.co_varnames,
        filename,
        co.co_name,
        co.co_firstlineno,
        co.co_lnotab,
        co.co_freevars,
        co.co_cellvars,
    ]
    if hasattr(co, "co_posonlyargcount"):
        # PEP570 added "positional only arguments" in Python 3.8
        params.insert(1, co.co_posonlyargcount)
    return types.CodeType(*params)



# relimport.py is a very simple script that tests importing using relative
# imports (available in Python 2.5 and up)
#
# Run the build process by running the command 'python setup.py build'
#
# If everything works well you should find a subdirectory in the build
# subdirectory that contains the files needed to run the script without Python

from cx_Freeze import setup, Executable

executables = [Executable("relimport.py")]

setup(
    name="relimport",
    version="0.1",
    description="Sample cx_Freeze script for relative imports",
    executables=executables,
)



"""sample to show GetDependentFiles in action - for Windows only"""

# This sample is a piece of code extracted from the freeze module,
# in order to demonstrate the dependencies of the specified file.
#
# Usage:
#   python getdependentfiles.py FILE
# Where FILE is a .exe, .pyd or .dll file
#
# See also: test_1.py - a batch of tests

import os
import sys

try:
    import cx_Freeze.util
except ImportError:
    print("Please install a cx-freeze package to test", file=sys.stderr)
    sys.exit(-1)


def print_usage():
    print("cx_Freeze dependencies demo (for windows only)", file=sys.stderr)
    print("usage:\n\t%s FILE..." % sys.argv[0], file=sys.stderr)
    return -1


def main():
    if len(sys.argv) <= 1:
        return print_usage()
    if sys.platform != "win32":
        print(sys.argv[0] + " is only for windows", file=sys.stderr)
        return -1
    res = 0
    for i in range(1, len(sys.argv)):
        path = sys.argv[i]
        print(path)
        try:
            dependent_files = cx_Freeze.util.GetDependentFiles(path)
        except cx_Freeze.util.BindError as exc:
            dependent_files = []
            print("error during GetDependentFiles() of ")
            print(f"{path!r}: {exc!s}", file=sys.stderr)
            res = -1
        if dependent_files:
            for file_name in dependent_files:
                print("\t", os.path.basename(file_name), "=>", file_name)
        else:
            print("\t(dependency not found)")
    return res


if __name__ == "__main__":
    sys.exit(main())



"""
A simple setup script for creating a Windows service.
See the comments in the Config.py and ServiceHandler.py files for more
information on how to set this up.

Installing the service is done with the option --install <Name> and
uninstalling the service is done with the option --uninstall <Name>. The
value for <Name> is intended to differentiate between different invocations
of the same service code -- for example for accessing different databases or
using different configuration files.
"""

from cx_Freeze import setup, Executable

options = {
    "build_exe": {
        "includes": ["ServiceHandler", "cx_Logging"],
        "excludes": ["tkinter"],
    }
}

executables = [
    Executable(
        "Config.py",
        base="Win32Service",
        targetName="cx_FreezeSampleService.exe",
    )
]

setup(
    name="cx_FreezeSampleService",
    version="0.1",
    description="Sample cx_Freeze Windows serice",
    executables=executables,
    options=options,
)
